local _G = _G
local RW = _G.RW3

local expect = RW.Debug.expect
local DEBUG = RW.DEBUG

local LDB = {}
RW.Common:NamedTable(LDB, "LDB")
RW.LDB = LDB

local ClickFunctions = {}
local ValidClickTypes = {
	["Left"] = true,
	["LeftShift"]  = true,
	["LeftAlt"] = true,
	["LeftCtrl"] = true,
	["Right"] = true,
	["RightShift"]  = true,
	["RightAlt"] = true,
	["RightCtrl"] = true,
}
for t in pairs(ValidClickTypes) do
	ClickFunctions[t] = {}
end

local ClickHelpLines = {}
local TooltipLines = {}

local function OnTooltipShow(tt)
	tt:AddLine("Raid Watch 3")
	
	for id, line in pairs(TooltipLines) do
		tt:AddLine(line)
	end	
	
	for i, line in pairs(ClickHelpLines) do
		tt:AddLine(line)
	end
end

local function OnClick(self, button)
	local ShiftKeyIsDown = IsShiftKeyDown()
	local AltKeyIsDown = IsAltKeyDown()
	local CtrlKeyIsDown = IsControlKeyDown()
	
	if button == "LeftButton" then
		if ShiftKeyIsDown then
			if ClickFunctions["LeftShift"] then ClickFunctions["LeftShift"]() end
		elseif AltKeyIsDown then
			if ClickFunctions["LeftAlt"] then ClickFunctions["LeftAlt"]() end
		elseif AltKeyIsDown then
			if ClickFunctions["LeftCtrl"] then ClickFunctions["LeftCtrl"]() end
		else
			for _, ID in pairs(ClickFunctions["Left"]) do
				ID()
			end
		end
		
	elseif button == "RightButton" then
		if ShiftKeyIsDown then
			if ClickFunctions["RightShift"] then ClickFunctions["RightShift"]() end
		elseif AltKeyIsDown then
			if ClickFunctions["RightAlt"] then ClickFunctions["RightAlt"]() end
		elseif AltKeyIsDown then
			if ClickFunctions["RightCtrl"] then ClickFunctions["RightCtrl"]() end
		else
			if ClickFunctions["Right"] then ClickFunctions["Right"]() end
		end
	end
end

function LDB:Init()
	RW.UnregisterCallback(self, "Enabled")

	local ldb = LibStub("LibDataBroker-1.1")
	if not ldb then return end
	local l = ldb:NewDataObject("Raid Watch 3")
	l.type = "data source"
	l.icon = [[Interface\Icons\INV_Misc_EngGizmos_01]]
	l.OnTooltipShow = OnTooltipShow
	l.OnClick = OnClick
	
	self.object = l
end

function LDB:SetSuffix(suffix)
	if DEBUG then
		expect(suffix, "typeof", "string;number")
	end
	self.object.suffix = suffix	
end

function LDB:SetValue(value)
	if DEBUG then
		expect(value, "typeof", "string;number")
	end
	self.object.value = value
end

function LDB:SetText(text)
	if DEBUG then
		expect(text, "typeof", "string;number")
	end
	self.object.text = text
end

function LDB:AddClickFunction(clickType, ID, func)
	if DEBUG then
		expect(clickType, "typeof", "string")
		expect(ID, "typeof", "string;number")
		expect(func, "typeof", "function")
	end
	
	if not ValidClickTypes[clickType] then
		error(("Bad clickType to `AddClickFunction': %q"):format(clickType), 2)
	end
	if ClickFunctions[clickType][ID] then
		error(("Click function for %q with ID %q allready exists"):format(clickType, ID), 2)
	end

	ClickFunctions[clickType][ID] = func
end

function LDB:RemoveClickFunction(clickType, ID)
	if DEBUG then
		expect(clickType, "typeof", "string")
		expect(ID, "typeof", "string;number")
	end
	
	if not ValidClickTypes[clickType] then
		error(("Bad clickType to `RemoveClickFunction': %q"):format(clickType), 2)
	end

	ClickFunctions[clickType][ID] = nil
end

function LDB:AddClickHelpLine(ID, line)
	if DEBUG then
		expect(line, "typeof", "string;number")
		expect(line, "typeof", "string;number")
	end
	
	ClickHelpLines[ID] = line
end

function LDB:RemoveClickHelpLine(ID)
	if DEBUG then
		expect(ID, "typeof", "string;number")
	end
	
	ClickHelpLines[ID] = nil
end

function LDB:AddTooltipLine(id, line)
	if DEBUG then
		expect(id, "typeof", "string")
		expect(line, "typeof", "string;number")
	end
	
	TooltipLines[id] = line
end

function LDB:RemoveTooltipLine(id)
	if DEBUG then
		expect(id, "typeof", "string")
	end
	
	TooltipLines[id] = nil
end

RW.RegisterCallback(LDB, "Enabled", "Init")