local _G = _G
local RW = _G.RW3

local Debug = RW.Debug
local expect = Debug.expect
local Common = RW.Common

local CI = LibStub("LibConfigUI-1.0")

local plugin = RW:NewModule("Config")
RW.Config = plugin

Debug:AddGlobal("Config", plugin)

local About = {
	name = "About",
	children = {
		{ 
			widget = "SimpleGroup",
			type = "group",
			layout = "Fill",
			children = {
				{ 	
					widget = "CI-DropdownGroup", 
					name = "Test Dropdown",
					type = "group",
					value = 34352,
					groups = {
						{ 
							widget = "CI-Dropdown-Item-Spell", value = 34352, text = "Value 1", 
							children = {
								{widget = "Label", text = "Test"},
							}
						},
						{ 
							widget = "CI-Dropdown-Item-Spell", value = 21342, text = "Value 1", 
							children = {
								{widget = "Label", text = "Test2"},
							}
						}
					}
				},
			}
		}
	}
}

local TopMenu = {
	widget = "CI-GridSelectGroup",
	type = "group",
	height = 30,
	layout = "Fill",
	groups = {
	}
}

local OptionsWindow
local OptionsWindowStatus = {}
local ConfigTable = {
	type = "group",
	widget = "CI-Frame",
	name = "Raid Watch 3",
	icon = [[Interface\Icons\INV_Misc_EngGizmos_01]],
	layout = "Fill",
	statustable = OptionsWindowStatus,
	width = 800,
	height = 600,
	children = {
		TopMenu,
	}
}

local Debug = {	
	name = "Debug",
	children = {
		{
			widget = "ScrollFrame",
			type = "group",
			layout = "Flow",
			db = function() return RW.db.profile end,
			children = {
				{ widget = "CI-InlineGroup", type = "group", layout = "Flow", name = "Options", fullwidth = true, template = "Group1",
					children = {
						{ widget = "CI-Slider", key = "DebugLevel", name = "Debug Level", min = 0, max = 10, step = 1, fullwidth = true },
						{ widget = "CI-ColorSelect", name = "Color Select afsosaf asfasf aslfkafs sf", width = 400, key = "Color" },
						{ 
							widget = "SimpleGroup", layout = "Flow", fullwidth = true, type="group",
							children = function()
								local r = {}
									for event, listeners in pairs(RW.Callbacks.events) do
										local e = { widget = "CI-InlineGroup", relwidth = 0.5, children = {}, name = event, type = "group", layout = "Flow", template = "Group2" }
										for listener in pairs(listeners) do
											tinsert(e.children, {widget = "Label", text = tostring(listener) })
										end
										tinsert(r, e)
									end
								return r
							end
						}
					},
				},
			},
		}
	},
}

function plugin:OnEnable()
	RW.LDB:AddClickFunction("Left", "ShowConfig", self.Toggle)	

	CI = CI:RegisterConfigScheme("RW3")
	self.CI = CI
	
	self:RegisterTemplates()
	
	RW.Callbacks:Fire("ConfigLoaded")
end

function plugin:Toggle()
	local self = plugin

	if OptionsWindow then
		OptionsWindow:Hide()
	else
		OptionsWindow = CI:RegisterTable(ConfigTable)
		OptionsWindow:Show()
		OptionsWindow:SetCallback("OnClose", function(self, event, ...)
			self:Release()
			OptionsWindow = nil
		end)
	end
end

function plugin:AddTopMenu(value, t)
	expect(value, "typeof", "string;number")
	expect(TopMenu.groups[value], "==", nil)
	
	TopMenu.groups[value] = t
end

plugin:AddTopMenu("About", About)
plugin:AddTopMenu("Debug", Debug)