LoadAddOn("LibLinq-1.0")

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch 3")

local plugin = RW:NewPlugin("SpellSpy")
local Debug = RW.Debug

Debug:AddGlobal("SpellSpy", plugin)

local LibLinq = LibStub("LibLinq-1.0")

local options
local db
local Current

local defaults = {
	profile = {
		Enabled = false,
		TrackFriendly = true,
		TrackHostile = true,
		TrackPlayer = true,
		
		Data = {
		}
	}
}

function plugin:OnInitialize()
	self.db = RW.db:RegisterNamespace(self:GetName(), defaults)
	db = self.db.profile
	
	plugin:SetEnabledState(db.Enabled)	
	
	RW.RegisterCallback(self, "ConfigLoaded")
end

function plugin:OnEnable()	
	RW.RegisterCallback(self, "CombatEvent")	
	
	RW.LDB:AddTooltipLine("SpellSpy", self:GetName().." enabled")
end

function plugin:OnDisable()
	RW.UnregisterCallback(self, "CombatEvent")
	RW.LDB:AddTooltipLine("SpellSpy", self:GetName().." disabled")
	
	Current = nil
end

function plugin:ConfigLoaded()
	RW.Config:AddPlugin("SpellSpy", options)
end

function plugin:CombatEvent(e, event, args)	
	if args:IsSourceHostile() and not db.TrackHostile then Debug:Print(10, "Ignoring Hostlile", event, args.sName) return end
	if not args:IsSourceHostile() and not db.TrackFriendly and not args:IsSourcePlayer() then 
		Debug:Print(10, "Ignoring Friendly", event, args.sName) 
		return 
	end	
	if args:IsSourcePlayer() and not db.TrackPlayer then Debug:Print(10, "Ignoring Player", event) return end
	
	if args.spellID then
		if not Current then
			Current = {
				timestamp = date(),
				Names = {}
			}
			tinsert(db.Data, Current)
		end
		
		local name = Current.Names[args.sName or "nill"] or {}
		name.Spells = name.Spells or {}
		
		local spell = name.Spells[args.spellID] or {}
		spell.Events = spell.Events or {}
		spell.Events[event] = spell.Events[event] or { }
		spell.Events[event].Targets = spell.Events[event].Targets or {}
		spell.Events[event].Targets[args.dName or "nil"] = spell.Events[event].Targets[args.dName or "nil"] or { Count = 0 }
		spell.Events[event].Targets[args.dName or "nil"].Count = spell.Events[event].Targets[args.dName or "nil"].Count + 1 or 1
		spell.Events[event].Targets[args.dName or "nil"].Amounts = spell.Events[event].Targets[args.dName or "nil"].Amounts or {}
		if args.amount then
			tinsert(spell.Events[event].Targets[args.dName or "nil"].Amounts, args.amount + (args.overkill or 0) + (args.absorbed or 0) + (args.overhealing or 0))	
		end
		
		name.Spells[args.spellID] = spell
		Current.Names[args.sName or "nill"] = name
	end
end

function plugin:ApplySettings()
	self = plugin
	
	self:ToggleEnabled(db.Enabled)
end

local function CreateSpellgroup(Spells)
	local c = {}
	for spell, spellData in pairs(Spells) do
		tinsert(c, {
			widget = "CI-Dropdown-Item-Spell",
			value = spellData.spell,		
			children = {
				{
					widget = "ScrollFrame", type = "group", layout = "Flow", fullwidth = true,
					children = {
						{
							widget = "CI-InlineGroup", name = "Events", fullwidth = true, layout = "Flow", type = "group", template = "Group1",
							children =  function()
								local r = {}
								for event, eventData in pairs(spellData.data.Events) do
									local eventgrop = {
										widget = "CI-InlineGroup",
										name = event,
										fullwidth = true,
										layout = "Flow",
										type = "group",
										template = "Group2",
										children = {}
									}
									
									for name, nameData in pairs(eventData.Targets) do
										tinsert(eventgrop.children, {
											widget = "Label",
											text = name,
											relwidth = 0.4,
										})
										tinsert(eventgrop.children, {
											widget = "Label",
											text = "Count: "..nameData.Count,
											relwidth = 0.2,
										})
										
										local average, count = 0, 0
										for i, amount in pairs(nameData.Amounts) do
											average = average + amount
											count = count + 1
										end								
										
										if average ~= 0 then
											average = average / count
										end
										tinsert(eventgrop.children, {
											widget = "Label",
											text = average ~= 0 and ("Average: %.1f"):format(average) or "",
											relwidth = 0.39,
										})
									end
									
									tinsert(r, eventgrop)
								end
								return r
							end,
						}
					}
				}
			}
		})
	end
	
	return c
end

local TextBox

local SpellTreeStatus = {}
options = {
	name = "Spell Spy",
	icon = [[Interface\Icons\Ability_Druid_Cyclone]],
	groups = {		
		{
			name = "Options",
			type = "group",			
			children = {
				{ 
					widget = "ScrollFrame",
					type = "group",
					db = function() return db end,
					callback = plugin.ApplySettings,
					children = {
						{
							widget = "CI-InlineGroup",
							name = "Options",
							type = "group",
							layout = "Flow",
							fullwidth = true,
							template = "Group1",
							db = function() return db end,
							callback = plugin.ApplySettings,
							children = {
								{ widget = "CI-CheckBox", name = "Enabled", relwidth = 1 },
								{ widget = "CI-CheckBox", name = "TrackFriendly", text = "Track friendly sources", relwidth = 0.5, desc = "Track spells that are casted by the player", },
								{ widget = "CI-CheckBox", name = "TrackHostile", text = "Track hostile sources", relwidth = 0.5 },
								{ widget = "CI-CheckBox", name = "TrackPlayer", text = "Track events by player", relwidth = 0.5 },
								{ widget = "CI-Slider", name = "Debug Level", min = 0, max = 10, step = 1, relwidth = 0.5 },
							}
						}						
					}
				}	
			}
		},
	},
	children = {		
		{
			widget = "SimpleGroup",
			layout = "Flow",
			type = "group",
			children = {
				{
					widget = "SimpleGroup",
					name = "",
					type = "group",
					layout = "Flow",
					fullwidth = true,
					children = {
						{ widget = "Button", name = "Delete", callback = function()
							local text = SpellTreeStatus.selected
							LibLinq.List.WrapTable(db.Data):RemoveWhere("x => x.timestamp == '"..text.."'")
						end},
						{ widget = "EditBox", name = "Name", oncreated = function(widget) TextBox = widget end},
					}
				},
				{
					widget = "CI-TreeGroup",
					name = "SpellTree",
					type = "group",
					layout = "Fill",
					fullwidth = true,
					fullheight = true,
					statustable = SpellTreeStatus,
					selectedchange = function(group)
						if TextBox then
							TextBox:SetText(("\001"):split(group))
						end
					end,
					groups = function()
						local r = {}
						for i, group in pairs(db.Data) do
							tinsert(r, {
								name = group.timestamp,
								type = "group",
								groups = function()
									local g = {}
									for name, nameData in pairs(group.Names) do
										tinsert(g, {
											name = name,
											type = "group",
											children = {
												{
													widget = "CI-DropdownGroup",
													type = "group",
													layout = "Fill",
													fullwidth = true,
													fullheight = true,
													groups = function()
														local spells = {}
														for spell, spellData in pairs(nameData.Spells) do
															tinsert(spells, {spell = spell, data = spellData})
														end
														return CreateSpellgroup(spells)
													end,												
												}
											},
										})
									end
									return g
								end,
								children = {
									{
										widget = "CI-DropdownGroup",
										type = "group",
										layout = "Fill",
										fullwidth = true,
										fullheight = true,
										groups = function()
											local spells = {}
											for name, nameData in pairs(group.Names) do
												for spell, spellData in pairs(nameData.Spells) do
													tinsert(spells, {spell = spell, data = spellData})
												end
											end
											return CreateSpellgroup(spells)
										end,												
									}
								}
							})
						end
						return r
					end,
				}
			}
		}
	}
}